@@ -1,19 +1,28 @@
 from sqlobject import col, dberrors
-from sqlobject.compat import PY2
+from sqlobject.compat import PY2, string_type
 from sqlobject.converters import registerConverter, StringLikeConverter
 from sqlobject.dbconnection import DBAPI
 
 
 class ErrorMessage(str):
     def __new__(cls, e, append_msg=''):
-        if len(e.args) > 1:
-            obj = str.__new__(cls, e.args[1] + append_msg)
+        if e.__module__ == 'cymysql.err':
+            if isinstance(e.errmsg, string_type):
+                errmsg = e.errmsg
+            else:
+                errmsg = e.errmsg.reason
+            errcode = e.errno
         else:
-            obj = str.__new__(cls, append_msg)
-        try:
-            obj.code = int(e.args[0])
-        except ValueError:
-            obj.code = e.args[0]
+            if len(e.args) > 1:
+                errmsg = e.args[1]
+            else:
+                errmsg = ''
+            try:
+                errcode = int(e.args[0])
+            except ValueError:
+                errcode = e.args[0]
+        obj = str.__new__(cls, errmsg + append_msg)
+        obj.code = errcode
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
@@ -32,23 +41,23 @@ class MySQLConnection(DBAPI):
     def __init__(self, db, user, password='', host='localhost', port=0, **kw):
         drivers = kw.pop('driver', None) or 'mysqldb'
         for driver in drivers.split(','):
-            driver = driver.strip()
+            driver = driver.strip().lower()
             if not driver:
                 continue
             try:
-                if driver.lower() in ('mysqldb', 'pymysql'):
-                    if driver.lower() == 'pymysql':
+                if driver in ('mysqldb', 'pymysql'):
+                    if driver == 'pymysql':
                         import pymysql
                         pymysql.install_as_MySQLdb()
                     import MySQLdb
-                    if driver.lower() == 'mysqldb':
+                    if driver == 'mysqldb':
                         if MySQLdb.version_info[:3] < (1, 2, 2):
                             raise ValueError(
                                 'SQLObject requires MySQLdb 1.2.2 or later')
                     import MySQLdb.constants.CR
                     import MySQLdb.constants.ER
                     self.module = MySQLdb
-                    if driver.lower() == 'mysqldb':
+                    if driver == 'mysqldb':
                         self.CR_SERVER_GONE_ERROR = \
                             MySQLdb.constants.CR.SERVER_GONE_ERROR
                         self.CR_SERVER_LOST = \
@@ -59,6 +68,16 @@ class MySQLConnection(DBAPI):
                         self.CR_SERVER_LOST = \
                             MySQLdb.constants.CR.CR_SERVER_LOST
                     self.ER_DUP_ENTRY = MySQLdb.constants.ER.DUP_ENTRY
+                elif driver == 'cymysql':
+                    import cymysql
+                    import cymysql.constants.CR
+                    import cymysql.constants.ER
+                    self.module = cymysql
+                    self.CR_SERVER_GONE_ERROR = \
+                        cymysql.constants.CR.CR_SERVER_GONE_ERROR
+                    self.CR_SERVER_LOST = \
+                        cymysql.constants.CR.CR_SERVER_LOST
+                    self.ER_DUP_ENTRY = cymysql.constants.ER.DUP_ENTRY
                 elif driver in ('connector', 'connector-python'):
                     import mysql.connector
                     self.module = mysql.connector
@@ -71,13 +90,6 @@ class MySQLConnection(DBAPI):
                         self.connector_type = 'mysql.connector-python'
                     else:
                         self.connector_type = 'mysql.connector'
-                elif driver == 'oursql':
-                    import oursql
-                    self.module = oursql
-                    self.CR_SERVER_GONE_ERROR = \
-                        oursql.errnos['CR_SERVER_GONE_ERROR']
-                    self.CR_SERVER_LOST = oursql.errnos['CR_SERVER_LOST']
-                    self.ER_DUP_ENTRY = oursql.errnos['ER_DUP_ENTRY']
                 elif driver == 'mariadb':
                     import mariadb
                     self.module = mariadb
@@ -97,7 +109,7 @@ class MySQLConnection(DBAPI):
                     raise ValueError(
                         'Unknown MySQL driver "%s", '
                         'expected mysqldb, connector, connector-python, '
-                        'oursql, pymysql, mariadb, '
+                        'pymysql, cymysql, mariadb, '
                         'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
@@ -149,12 +161,6 @@ class MySQLConnection(DBAPI):
                                     db, host, port, user, password
                                     )
 
-        elif driver == 'oursql':
-            if "use_unicode" not in self.kw:
-                self.kw["use_unicode"] = not PY2
-            # oursql doesn't implement ping(True) yet
-            self.kw["autoreconnect"] = True
-
         elif driver == 'mariadb':
             self.kw.pop("charset", None)
 
@@ -182,7 +188,7 @@ class MySQLConnection(DBAPI):
     def makeConnection(self):
         dbEncoding = self.dbEncoding
         if dbEncoding:
-            if self.driver.lower() in ('mysqldb', 'pymysql'):
+            if self.driver in ('mysqldb', 'pymysql'):
                 from MySQLdb.connections import Connection
                 if not hasattr(Connection, 'set_character_set'):
                     # monkeypatch pre MySQLdb 1.2.1
@@ -206,7 +212,7 @@ class MySQLConnection(DBAPI):
                     # mariadb doesn't implement ping(True)
                     conn.auto_reconnect = True
                     conn.ping()
-                elif self.driver != 'oursql':
+                else:
                     # Attempt to reconnect. This setting is persistent.
                     conn.ping(True)
         except self.module.OperationalError as e:
@@ -228,8 +234,6 @@ class MySQLConnection(DBAPI):
                     conn.setencoding(encoding=dbEncoding)
             elif hasattr(conn, 'set_character_set'):
                 conn.set_character_set(dbEncoding)
-            elif self.driver == 'oursql':
-                conn.charset = dbEncoding
             elif hasattr(conn, 'query'):
                 # works along with monkeypatching code above
                 conn.query("SET NAMES %s" % dbEncoding)
@@ -245,7 +249,7 @@ class MySQLConnection(DBAPI):
                 conn.autocommit = auto
 
     def _force_reconnect(self, conn):
-        if self.driver.lower() == 'pymysql':
+        if self.driver in ('pymysql', 'cymysql'):
             conn.ping(True)
             self._setAutoCommit(conn, bool(self.autoCommit))
             if self.dbEncoding:
@@ -257,7 +261,7 @@ class MySQLConnection(DBAPI):
         dbEncoding = self.dbEncoding
         if dbEncoding and not isinstance(query, bytes) and (
                 self.driver in ('mysqldb', 'connector', 'connector-python',
-                                'oursql', 'mariadb')):
+                                'mariadb')):
             query = query.encode(dbEncoding, 'surrogateescape')
         # When a server connection is lost and a query is attempted, most of
         # the time the query will raise a SERVER_LOST exception, then at the
@@ -271,7 +275,7 @@ class MySQLConnection(DBAPI):
         # reconnect flag must be set when making the connection to indicate
         # that autoreconnecting is desired. In MySQLdb 1.2.2 or newer this is
         # done by calling ping(True) on the connection.
-        # PyMySQL needs explicit reconnect
+        # [PC]yMySQL need explicit reconnect
         # each time we detect connection timeout.
         for count in range(3):
             try:
@@ -283,7 +287,7 @@ class MySQLConnection(DBAPI):
                         raise dberrors.OperationalError(ErrorMessage(e))
                     if self.debug:
                         self.printDebug(conn, str(e), 'ERROR')
-                    if self.driver.lower() == 'pymysql':
+                    if self.driver in ('pymysql', 'cymysql'):
                         self._force_reconnect(conn)
                 else:
                     raise dberrors.OperationalError(ErrorMessage(e))
@@ -359,9 +363,21 @@ class MySQLConnection(DBAPI):
         return index.mysqlCreateIndexSQL(soClass)
 
     def createIDColumn(self, soClass):
-        if soClass.sqlmeta.idType == str:
+        if soClass.sqlmeta.idType is str:
             return '%s TEXT PRIMARY KEY' % soClass.sqlmeta.idName
-        return '%s INT PRIMARY KEY AUTO_INCREMENT' % soClass.sqlmeta.idName
+        if soClass.sqlmeta.idType is not int:
+            raise TypeError('sqlmeta.idType must be int or str, not %r'
+                            % soClass.sqlmeta.idType)
+        if soClass.sqlmeta.idSize is None:
+            mysql_int_type = 'INT'
+        elif soClass.sqlmeta.idSize in ('TINY', 'SMALL', 'MEDIUM', 'BIG'):
+            mysql_int_type = '%sINT' % soClass.sqlmeta.idSize
+        else:
+            raise ValueError(
+                "sqlmeta.idSize must be 'TINY', 'SMALL', 'MEDIUM', 'BIG' "
+                "or None, not %r" % soClass.sqlmeta.idSize)
+        return '%s %s PRIMARY KEY AUTO_INCREMENT' \
+               % (soClass.sqlmeta.idName, mysql_int_type)
 
     def joinSQLType(self, join):
         return 'INT NOT NULL'
