@@ -21,6 +21,9 @@ class ErrorMessage(str):
             # pg8000 for Python 3.5+
             ecode = eargs0['C']
             eerror = emessage = eargs0['M']
+        elif e.__module__ == 'pg':  # PyGreSQL
+            ecode = e.sqlstate
+            eerror = emessage = e.args[0]
         elif hasattr(e, 'pgcode'):  # psycopg2 or psycopg2.errors
             ecode = getattr(e, 'pgcode', None)
             eerror = getattr(e, 'pgerror', None)
@@ -60,9 +63,12 @@ class PostgresConnection(DBAPI):
             if not driver:
                 continue
             try:
-                if driver in ('psycopg', 'psycopg2'):
-                    import psycopg2 as psycopg
+                if driver == 'psycopg':
+                    import psycopg
                     self.module = psycopg
+                elif driver == 'psycopg2':
+                    import psycopg2
+                    self.module = psycopg2
                 elif driver == 'pygresql':
                     import pgdb
                     self.module = pgdb
@@ -133,7 +139,10 @@ class PostgresConnection(DBAPI):
                 else:
                     dsn_dict["port"] = port
             if db:
-                dsn_dict["database"] = db
+                if driver == 'psycopg':
+                    dsn_dict["dbname"] = db
+                else:
+                    dsn_dict["database"] = db
             if user:
                 dsn_dict["user"] = user
             if password:
@@ -341,7 +350,22 @@ class PostgresConnection(DBAPI):
         return index.postgresCreateIndexSQL(soClass)
 
     def createIDColumn(self, soClass):
-        key_type = {int: "SERIAL", str: "TEXT"}[soClass.sqlmeta.idType]
+        if soClass.sqlmeta.idType is int:
+            if soClass.sqlmeta.idSize in ('TINY', 'SMALL'):
+                key_type = 'SMALLSERIAL'
+            elif soClass.sqlmeta.idSize in ('MEDIUM', None):
+                key_type = 'SERIAL'
+            elif soClass.sqlmeta.idSize == 'BIG':
+                key_type = 'BIGSERIAL'
+            else:
+                raise ValueError(
+                    "sqlmeta.idSize must be 'TINY', 'SMALL', 'MEDIUM', 'BIG' "
+                    "or None, not %r" % soClass.sqlmeta.idSize)
+        elif soClass.sqlmeta.idType is str:
+            key_type = "TEXT"
+        else:
+            raise TypeError('sqlmeta.idType must be int or str, not %r'
+                            % soClass.sqlmeta.idType)
         return '%s %s PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def dropTable(self, tableName, cascade=False):
